%% Set Parameters for the full system 
%   potential      V(x1,x2) = fnA(x1) + lambda* |x2-fnB(x1)|^2
%   Gradient system    dX = -grad(V)dt + beta dW 
%      dx1 = - grad(fnA)(x1) dt + lambda*<x2-fnB(x1),grad(fnB(x1))> dt + sigma_1 dW_1(t)
%      dx2 = - lambda[x2- B(x1)]dt + sigma_2 dW_2(t)

function [sysInfo,fmDataInfo] = model_settings(SAVE_DIR)
 
scheme   = 'Backward_Euler'; % 'Euler'; 'Backward_Euler'


%mu = 0.1; beta = 1; omega = 6; lambda = 10;  
mu = 1; beta = 1; omega = 3*pi; lambda = 10;  % potential(x,y) = mu/2 * fnA(x) + lambda/2* |y - fnB(x)|^2
potential_type = 'Ax2Bsin';    % 'A1B1'  
switch potential_type 
    case 'Ax2Bsin'       % 
        fnA         = @(x)  mu*x.^2/2;  fnB   = @(x) sin(omega*x);
        gradA       = @(x)  mu*x;       gradB = @(x) omega*cos(omega*x);  
        % need to be corrected in high dimension case
        hessA       = @(x)  mu;         hessB = @(x) -omega.^2.*sin(omega*x);
        potential_V = @(x,y) fnA(x)+ lambda/2* (y-fnB(x)).^2;
        dVx         = @(x,y) gradA(x) - lambda* (y-fnB(x)).*gradB(x);  % note the .* OK for matrix
        dVy         = @(x,y) lambda* (y-fnB(x));
        % need to be corrected in high dimension case
        dVxx        = @(x,y) hessA(x)+lambda*(gradB(x)).*(gradB(x))-lambda*(y-fnB(x)).*hessB(x);
        dVxy        = @(x,y) lambda*(-gradB(x));
        dVyy        = @(x,y) lambda;
        gradV       = @(X) [dVx(X(1,:),X(2,:)); dVy(X(1,:),X(2,:))];
        HessV       = @(X) [dVxx(X(1,:),X(2,:)), dVxy(X(1,:),X(2,:));  dVxy(X(1,:),X(2,:)), dVyy(X(1,:),X(2,:))];
        LaplaceGradV= @(X) [lambda*omega^3* ( X(2,:).*cos(omega*X(1,:)) - 2*cos(2*omega*X(1,:))); ...
                              lambda*omega^2*sin(omega*X(1,:))];                
end

 xgrid = -4:0.1:4; ygrid = -5:0.1:5; 
 plotPotential(potential_V,xgrid,ygrid); 

% information of the true system 
sysInfo = struct(...       
       'potential_type', potential_type,...
       'potential_V',  potential_V,...
       'mu',           mu,...
       'beta',         beta,...
       'omega',        omega,...
       'sigma_1',      sqrt(2/beta),...
       'sigma_2',      sqrt(2/beta),...
       'lambda',       lambda,...       
       'fnA',          fnA,...
       'fnB',          fnB, ...
       'gradA',        gradA,...
       'gradB',        gradB,...
       'hessA',        hessA,...
       'hessB',        hessB,...
       'xgrid', xgrid,...
       'ygrid', ygrid, ...
       'gradV',        gradV,...
       'HessV',        HessV,...
       'LaplaceGradV', LaplaceGradV ...
   );

%% setting of the numerical scheme for the true system==>> full model
M        = 1; tN = 5*1e6; dt = 0.0001; obsgap = 1; Init_distr = 'Gaussian'; 
betaLambda = sprintf('beta%ilambda%iM%i_tN%iw',beta,lambda, M,ceil(tN/10000)); 
% scheme   = 'Backward_Euler'; % 'Euler'; 'Backward_Euler'
switch scheme
    case 'Backward_Euler'; ImpSolver= 'Newton'; Newton_maxIte = 2000; Newton_stopCond = dt^2/4;
        fm_type  = [potential_type,'_',betaLambda,'_impEuler']; 
    case 'Euler'; ImpSolver= 'NA'; Newton_maxIte = 0; Newton_stopCond = 0;
        fm_type  = [potential_type,'_',betaLambda,'_expEuler']; 
end

fmDataInfo   = struct(...
    'scheme',  scheme,...
    'ImpSolver', ImpSolver, ...
    'dt',      dt,... 
    'tN',      tN,...
    'Newton_maxIte', Newton_maxIte,...
    'Newton_stopCond', Newton_stopCond,...
    'obsgap',  obsgap,...
    'M',       M,...        % number of trajectories 
    'type',    fm_type, ... % string for file names
    'dVx',     dVx,...
    'dVy',     dVy, ...
    'dVxx',    dVxx, ...
    'dVxy',    dVxy, ...
    'dVyy',    dVyy, ...
    'betaLambda', betaLambda,...
    'getInitial', @(M) getInitial(2,M,Init_distr) ... % randn(2,M);
     );


%% set DIR for saving data
datapath           = [SAVE_DIR,potential_type, '/',betaLambda, '/'];
if ~exist(datapath,'dir'), mkdir(datapath); end
datafileName       = [datapath,'Data_',fm_type,'.mat'];
fmDataInfo.datapath     = datapath; 
fmDataInfo.datafileName = datafileName;  

end


function init = getInitial(d,M,distr)
% generate samples from initial disribution
switch distr
    case 'Gaussian'; init = randn(d,M);
    case 'Unif';     init = rand(d,M); 
end
end

function plotPotential(potential_V,xgrid,ygrid)
% plot the potential function 

[X,Y] = meshgrid(xgrid,ygrid); 
fnVal = potential_V(X,Y);
surf(X,Y,fnVal); colorbar
end
