function [data_orig,X] = BAOAB(sysInfo,fmDataInfo,dWdt,X_init)
% solve the Hamiltonian system by the time-splitting BAOAB scheme:
%{
 Reference paper: 
Rational Construction of Stochastic Numerical Methods for Molecular Sampling     
 Output
    data_orig
%}
%%%--------------------
potential_type = sysInfo.potential_type;


mu    =   sysInfo.mu;
gamma =   sysInfo.gamma;
beta  =   sysInfo.beta;
M     =   sysInfo.M; 

inv_M  =   inv(M); % inverse of matrix M
sqrtM =   sqrt(M); % TO DO: double check MATLAB calculation of M^{1/2}
%%%--------------------
dimX        = sysInfo.dimX; % this addition 1 is used to take care of the momentum p

% drift_neg   = sysInfo.gradV;
% grad_drift_neg = sysInfo.hessV;
% diffMat     = sysInfo.diffusion;  % constant diffusion term
%eyeMatrix   = eye(dimX,dimX); % identity matrix used in the Newton's iteration.

dt    =   fmDataInfo.dt;
tN    =   fmDataInfo.tN;
dVx   =   fmDataInfo.dVx;

c1=exp(-gamma*dt);
c2=1/gamma*(1-c1);
c3=sqrt(1/beta*(1-c1^2))/sqrt(dt); % c3 is rescaled by sqrt(dt) due to we generate dWdt


%% initialize the system and generate the Brownian increments 
if nargin <4
    X_init =   fmDataInfo.getInitial(1);
end
if ~exist('dWdt','var')
    dWdt   =   sqrt(dt)*randn(dimX,tN);  % i.i.d increments of Bm for the integrator.
end

% X(1, :) store the position of particle
% P(1, :) store the momentum of particle
X      = zeros(dimX,tN+1);
P      = X;
X(:,1) = X_init;
x0     = X_init;


for i  = 1:tN
    
    %% Time-splitting BAOAB method
    % BA step
    P_half = P(1:dimX,i)-dt*dVx(X(1:dimX,i))/2;
    x_half = X(1:dimX,i)+dt*inv_M*P_half/2;
    % O step
    P_half_corr=c1*P_half+c3*sqrtM*dWdt(1:dimX,i);
    % AB step
    X(1:dimX,i+1)=x_half+dt*inv_M*P_half_corr/2;
    P(1:dimX,i+1)=P_half_corr-dt*dVx(X(1:dimX,i+1))/2;
    
    
end

data_orig = struct( ...
    'X',        X,...
    'P',        P,...
    'dWdt',     dWdt, ...
    'tN',       tN, ...
    'dt',       dt ...
    );
end
