function [data_orig,X] = SplitStepBackwardEuler_backupv2(sysInfo,fmDataInfo,X_init,dWdt)
% solve the system by the split step backward Euler integrator: 
%{ 
        dX = f(X)dt + b(X)dW 
 With  
step 1: X(:,*) = X(:,n)+ f(X(:,*))*dt
step 2: X(:,n+1) = X(:,*)+ b(X(:,*))*sqrt(dt) * W_1
 Output
    data_orig  
%}

dimX        = sysInfo.dimX; 
drift_neg   = sysInfo.gradV;   
grad_drift_neg = sysInfo.hessV;
diffMat     = sysInfo.diffusion;  % constant diffusion term
potential_type = sysInfo.potential_type;

 if strcmp(potential_type, 'LangLocal')  %  
        mu    =   sysInfo.mu;
        gamma =   sysInfo.gamma;
 end


eyeMatrix   = eye(dimX,dimX); % identity matrix used in the Newton's iteration.

dt    =   fmDataInfo.dt;  
tN    =   fmDataInfo.tN;
Newton_maxIte = fmDataInfo.Newton_maxIte;
Newton_stopCond = fmDataInfo.Newton_stopCond;

if nargin <3
    X_init =   fmDataInfo.getInitial(1);
end
if ~exist('dWdt','var')  
    dWdt   =   sqrt(dt)*randn(dimX,tN);  % i.i.d increments of Bm for the integrator.
end

X      = zeros(dimX,tN+1);
X(:,1) = X_init;
x0     = X_init; 

Eig_Newton  = zeros(2,tN+1);
for i  = 1:tN
      
    % Implement the first step of Backward Eulter method
    % The implicit system is solved by Newton's method
    
    % initialize the Newton method: 
    ite      = 0;
    % current position of (x, y)
    X_curr   = X(:,i); 
    X_newton = X_curr; 
    % compute the vector-valed function F(X)=0 of the implicit system
    F = X_newton-X_curr+dt*drift_neg(X_newton);
    
    % compute the gradient of F(X)
    grad_F=eyeMatrix+dt*grad_drift_neg(X_newton);
    
    f_res=100; % initialize the residual of Newton's iteration;
   
    %% compute the max and min eigenvalues of grad_f using the quadratic roots formula
    
    switch dimX
        case 1
            %% dim==1 case
            % store the max and min eigenvalues of grad_f
            grad_F_eig_max = grad_drift_neg(X_curr);
            grad_F_eig_min = grad_drift_neg(X_curr);
            
        case 2
            %% dim==2 case
            temp_matrix=grad_drift_neg(X_curr); % a 2X2 matrix
            temp_b=2+dt*(temp_matrix(1,1)+temp_matrix(2,2)); 
            temp_square=dt*sqrt((temp_matrix(1,1)-temp_matrix(2,2))^2+4*(temp_matrix(1,2))^2);
            grad_F_eig_max=((temp_b)+temp_square)/2;
            grad_F_eig_min=((temp_b)-temp_square)/2;
    end
  
    
    
    if strcmp(potential_type, 'LangLocal')  %  
        %% First eliminate X(2,i+1), leaving a cubic for X(1,i+1), take the real root by Cardano's Formula
        tempa = mu*dt^2/(1+gamma*dt);
        tempc = (1-tempa);
        tempd = -X_curr(1,1)-dt/(1+gamma*dt)*X_curr(2,1); 
        tempQ = tempc/(3*tempa); tempR=-tempd/(2*tempa);
        tempD = tempQ^3+tempR^2;sqrtTempD=sqrt(tempD);
        tempS = nthroot((tempR+sqrtTempD),3);    tempT=nthroot((tempR-sqrtTempD),3);
        
        X_newton(1,1)=tempS+tempT;
        X_newton(2,1)=X_curr(2,1)/(1+gamma*dt)-mu*dt/(1+gamma*dt)*( X_newton(1,1)^3- X_newton(1,1));
        
    else
        while (f_res> Newton_stopCond ) && (ite<= Newton_maxIte)
            
            ite=ite+1;
            temp=X_newton-grad_F\F;
            X_newton=temp;
            % compute the vector-valed function F(X)=0 of the implicit system
            F = X_newton-X_curr+dt*drift_neg(X_newton);
            % compute the gradient of F(X)
            grad_F=eyeMatrix+dt*grad_drift_neg(X_newton);        
            %% store Hessian and eigenvalue and Hessian on/off at chosen time grid            
            % update the residual of Newton's method
            f_res=max(abs(F));
        end
        
        if (f_res> Newton_stopCond )
            disp('Newton method does not converge')
        end
    end
    
    %% Implement the second step of Backward Eulter method when diffusion matrix is constant
    X(:,i+1) = X_newton+diffMat*dWdt(:,i); 
        
    Eig_Newton(1, i)=grad_F_eig_max;
    Eig_Newton(2, i)=grad_F_eig_min;
    
    
end

data_orig = struct( ...
    'X',        X,...
    'dWdt',     dWdt, ...
    'tN',       tN, ...
    'dt',       dt, ...
    'Eig_Newton',  Eig_Newton ...
);
end
